-- fourbit adder; behavioral (or functional) description; 
-- instead of having to create an adder entity for every possible width, you
-- can make the adder width generic; "width" can be defined as any positive 
-- value when the adder is instantiated; note that, the ":= 4" defines a 
-- default value of 4 for the width if no value is assigned when the adder 
-- is instantiated;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity add is
  generic (
    width : positive := 4);
  port (
    in1   : in  std_logic_vector(width-1 downto 0);
    in2   : in  std_logic_vector(width-1 downto 0);
    sum   : out std_logic_vector(width-1 downto 0);
    carry : out std_logic
  );
end add;


architecture BHV1 of add is

begin

  process(in1, in2)
    variable temp : unsigned(width downto 0);
  begin
    temp := resize(unsigned(in1), width+1)+resize(unsigned(in2), width+1);
    sum   <= std_logic_vector(temp(width-1 downto 0));    
    carry <= temp(width);
  end process;
end BHV1;


architecture BHV2 of add is

  signal temp : unsigned(width downto 0);

begin 
  temp  <= resize(unsigned(in1), width+1)+resize(unsigned(in2), width+1);
  sum   <= std_logic_vector(temp(width-1 downto 0));    
  carry <= temp(width);
end BHV2;
